@@ -1,21 +1,13 @@
 /*
- * $Id: evdev.c,v 1.3 2005/11/18 18:12:29 pingc Exp $
+ * $Id: evdev.c,v 1.7 2006/05/05 17:25:02 pingc Exp $
  *
  *  Copyright (c) 1999-2001 Vojtech Pavlik
- *  Copyright (c) 2003 John Joganic         <john@joganic.com>
  *
  *  Event char devices, giving access to raw input device events.
  *
  *  Sponsored by SuSE
- *
- *  Revision History - AS BRANCH FROM LINUX WACOM PROJECT
- *  2003-03-06  2.4.20-j0.5.0  - Added to Linux Wacom project
- *                               Applied Jonathan Paisley's oops patch
- *  2003-04-02  2.4.20-j0.5.1  - Added J. Paisley's reattach patch
  */
 
-#define EVDEV_VERSION "2.4.20-j0.5.1"
-
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -241,7 +233,7 @@ static int evdev_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
 	int retval;
 
 	if (!evdev->exist)
-				return -ENODEV;
+		return -ENODEV;
 
 	switch (cmd) {
 
@@ -353,20 +345,19 @@ static struct input_handle *evdev_connect(struct input_handler *handler, struct
 	for (minor = 0; minor < EVDEV_MINORS && evdev_table[minor]; minor++)
 	{
 
-            if (!evdev_table[minor]->exist && evdev_table[minor]->open) /* reuse */ 
-            {
-                evdev = evdev_table[minor];
-                evdev->handle.dev = dev;
-                evdev->handle.handler = handler;
-                evdev->handle.private = evdev; /* should already be */
-                evdev->exist = 1;
-                printk(KERN_INFO "evdev: reusing event%d for input%d\n", minor, dev->number);
-		input_open_device(&evdev->handle);
-		wake_up_interruptible(&evdev->wait);
-                return &evdev->handle;
-            }
+		if (!evdev_table[minor]->exist && evdev_table[minor]->open) /* reuse */
+		{
+			evdev = evdev_table[minor];
+			evdev->handle.dev = dev;
+			evdev->handle.handler = handler;
+			evdev->handle.private = evdev; /* should already be */
+			evdev->exist = 1;
+			printk(KERN_INFO "evdev: reusing event%d for input%d\n", minor, dev->number);
+			input_open_device(&evdev->handle);
+			wake_up_interruptible(&evdev->wait);
+			return &evdev->handle;
+		}
 	}
-
 	if (minor == EVDEV_MINORS) {
 		printk(KERN_ERR "evdev: no more free evdev devices\n");
 		return NULL;
@@ -420,7 +411,6 @@ static struct input_handler evdev_handler = {
 
 static int __init evdev_init(void)
 {
-	printk(KERN_INFO "evdev.c: version " EVDEV_VERSION "\n");
 	input_register_handler(&evdev_handler);
 	return 0;
 }
@@ -434,10 +424,6 @@ module_init(evdev_init);
 module_exit(evdev_exit);
 
 MODULE_AUTHOR("Vojtech Pavlik <vojtech@suse.cz>");
-#ifndef __JEJ_DEBUG
-MODULE_DESCRIPTION("Event character device driver (LINUXWACOM)");
-#else
-MODULE_DESCRIPTION("Event character device driver (LINUXWACOM-DEBUG)");
-#endif
+MODULE_DESCRIPTION("Event character device driver");
 MODULE_LICENSE("GPL");
 
