@@ -1,5 +1,5 @@
 /*
- * $Id: wacom.c,v 1.5 2005/11/18 18:12:29 pingc Exp $
+ * $Id: wacom.c,v 1.8 2006/05/05 17:25:02 pingc Exp $
  *
  *  Copyright (c) 2000-2002 Vojtech Pavlik  <vojtech@suse.cz>
  *  Copyright (c) 2000 Andreas Bach Aaen    <abach@stofanet.dk>
@@ -9,7 +9,7 @@
  *  Copyright (c) 2000 Daniel Egger         <egger@suse.de>
  *  Copyright (c) 2001 Frederic Lepied      <flepied@mandrakesoft.com>
  *  Copyright (c) 2002 Christer Nilsson     <christer.nilsson@kretskompaniet.se>
- *  Copyright (c) 2002-2005 Ping Cheng      <pingc@wacom.com>
+ *  Copyright (c) 2002-2006 Ping Cheng      <pingc@wacom.com>
  *  Copyright (c) 2002 John Joganic         <john@joganic.com>
  *
  *  USB Wacom Graphire and Intuos tablet support
@@ -56,6 +56,7 @@
  *    v2.4.30-pc0.6.9   - Supports x86_64 systems
  *    v2.4.30-pc0.7.0   - Added Volito2 support
  *    v2.4.30-pc0.7.1   - Added DTF720, DTU 710, G4
+ *    v2.4.30-pc0.7.3   - Added DTF 521, I3 12x12, and I3 12x19
  */
 
 /*
@@ -86,7 +87,7 @@
 /*
  * Version Information
  */
-#define DRIVER_VERSION "v2.4.30-pc0.7.1"
+#define DRIVER_VERSION "v2.4.30-pc0.7.3"
 #define DRIVER_AUTHOR "Vojtech Pavlik <vojtech@suse.cz>"
 #ifndef __JEJ_DEBUG
 #define DRIVER_DESC "USB Wacom Graphire and Wacom Intuos tablet driver (LINUXWACOM)"
@@ -256,25 +257,44 @@ static void wacom_penpartner_irq(struct urb *urb)
 	struct wacom *wacom = urb->context;
 	unsigned char *data = wacom->data;
 	struct input_dev *dev = &wacom->dev;
-	int leftmb = (((signed char)data[6] > -80) && !(data[5] &0x20));
 
 	if (urb->status) return;
 
-	if (data[0] != 2 && data[0] != 5)
-	{
-		printk(KERN_INFO "wacom_penpartner_irq: received unknown report #%d\n", data[0]);
-		return;
-	}
-
-	input_report_key(dev, BTN_TOOL_PEN, 1);
-	input_report_abs(dev, ABS_MISC, STYLUS_DEVICE_ID); /* report tool id */
-	input_report_abs(dev, ABS_X, data[2] << 8 | data[1]);
-	input_report_abs(dev, ABS_Y, data[4] << 8 | data[3]);
-	input_report_abs(dev, ABS_PRESSURE, (signed char)data[6] + 127);
-	input_report_key(dev, BTN_TOUCH, leftmb);
-	input_report_key(dev, BTN_STYLUS, (data[5] & 0x40));
-
-	input_event(dev, EV_MSC, MSC_SERIAL, leftmb);
+	switch (data[0]) {
+		case 1:
+			if (data[5] & 0x80) {
+				wacom->tool[0] = (data[5] & 0x20) ? BTN_TOOL_RUBBER : BTN_TOOL_PEN;
+				wacom->id[0] = (data[5] & 0x20) ? ERASER_DEVICE_ID : STYLUS_DEVICE_ID;
+				input_report_key(dev, wacom->tool[0], 1);
+				input_report_abs(dev, ABS_MISC, wacom->id[0]); /* report tool id */
+				input_report_abs(dev, ABS_X, data[2] << 8 | data[1]));
+				input_report_abs(dev, ABS_Y, data[4] << 8 | data[3]));
+				input_report_abs(dev, ABS_PRESSURE, (signed char)data[6] + 127);
+				input_report_key(dev, BTN_TOUCH, ((signed char)data[6] > -127));
+				input_report_key(dev, BTN_STYLUS, (data[5] & 0x40));
+			} else {
+				input_report_key(dev, wacom->tool[0], 0);
+				input_report_abs(dev, ABS_MISC, 0); /* report tool id */
+				input_report_abs(dev, ABS_PRESSURE, -1);
+				input_report_key(dev, BTN_TOUCH, 0);
+			}
+			input_event(dev, EV_MSC, MSC_SERIAL, data[5] & 0x80);
+			break;
+		case 2:
+			input_report_key(dev, BTN_TOOL_PEN, 1);
+			input_report_abs(dev, ABS_MISC, STYLUS_DEVICE_ID); /* report tool id */
+			input_report_abs(dev, ABS_X, data[2] << 8 | data[1]);
+			input_report_abs(dev, ABS_Y, data[4] << 8 | data[3]);
+			input_report_abs(dev, ABS_PRESSURE, (signed char)data[6] + 127);
+			input_report_key(dev, BTN_TOUCH, ((signed char)data[6] > -80) && !(data[5] & 0x20));
+			input_report_key(dev, BTN_STYLUS, (data[5] & 0x40));
+			input_event(dev, EV_MSC, MSC_SERIAL, ((signed char)data[6] > -80) && !(data[5] & 0x20));
+			break;
+		default:
+			printk(KERN_INFO "wacom_penpartner_irq: received unknown report #%d\n", data[0]);
+			wacom_request_reset(wacom);
+			return;
+        }
 }
 
 static void wacom_graphire_irq(struct urb *urb)
@@ -286,6 +306,8 @@ static void wacom_graphire_irq(struct urb *urb)
 
 	if (urb->status) return;
 
+	if (data[0] == 99) return; /* for Volito tablets */
+
 	if (data[0] != 2)
 	{
 		printk(KERN_INFO "wacom_graphire_irq: received unknown report #%d\n", data[0]);
@@ -309,10 +331,10 @@ static void wacom_graphire_irq(struct urb *urb)
 			case 2: /* Mouse with wheel */
 				input_report_key(dev, BTN_MIDDLE, data[1] & 0x04);
 				if ( strstr(wacom->features->name, "Graphire4") ) {
-					rw = data[7] & 0x04 ? -(data[7] & 0x03) : data[7] & 0x03;
-					input_report_rel(dev, REL_WHEEL, rw);
+					rw = data[7] & 0x04 ? (data[7] & 0x03)-4 : data[7] & 0x03;
+					input_report_rel(dev, REL_WHEEL, -rw);
 				} else
-					input_report_rel(dev, REL_WHEEL, (signed char) data[6]);
+					input_report_rel(dev, REL_WHEEL, -(signed char) data[6]);
 				/* fall through */
 
 			case 3: /* Mouse without wheel */
@@ -351,34 +373,19 @@ static void wacom_graphire_irq(struct urb *urb)
 	/* send pad data */
 	if ( strstr(wacom->features->name, "Graphire4") )
 	{
-		/* fist tine sending pad data */
-		if (wacom->tool[1] != BTN_TOOL_FINGER)
-		{
-			wacom->id[1] = 0;
-			wacom->serial[1] = ( data[7] & 0x38 ) >> 2;
-		}
-		if ( data[7] & 0xf8 )
+		if ( (wacom->serial[1] & 0xc0) != (data[7] & 0xf8) )
 		{
+			wacom->id[1] = 1;
+			wacom->serial[1] = (data[7] & 0xf8);
 			input_report_key(dev, BTN_0, (data[7] & 0x40));
 			input_report_key(dev, BTN_4, (data[7] & 0x80));
-			if ( (( data[7] & 0x38 ) >> 2) == (wacom->serial[1] & 0x0e) )
-				/* alter REL_WHEEL value so X apps can get it */
-				wacom->serial[1] += (wacom->serial[1] & 0x01) ? -1 : 1 ;
-			else 
-				 wacom->serial[1] = (data[7] & 0x38 ) >> 2;
-			/* don't alter the value when there is no wheel event */
-			if ( wacom->serial[1] == 1 ) wacom->serial[1] = 0;
-			rw = wacom->serial[1];
-			rw = ( rw & 0x08 ) ? -( rw & 0x07 ) : ( rw & 0x07 ); 
+			rw = ((data[7] & 0x18) >> 3) - ((data[7] & 0x20) >> 3);
 			input_report_rel(dev, REL_WHEEL, rw);
-			wacom->tool[1] = BTN_TOOL_FINGER;
-			wacom->id[1] = data[7] & 0xf8;
-			input_report_key(dev, wacom->tool[1], 0xf0);
+			input_report_key(dev, BTN_TOOL_FINGER, 0xf0);
 			input_event(dev, EV_MSC, MSC_SERIAL, 0xf0);
 		} else if ( wacom->id[1] ) {
 			wacom->id[1] = 0;
-			wacom->serial[1] = 0;
-			input_report_key(dev, wacom->tool[1], 0);
+			input_report_key(dev, BTN_TOOL_FINGER, 0);
 			input_event(dev, EV_MSC, MSC_SERIAL, 0xf0);
 		}
 	}
@@ -457,9 +464,14 @@ static int wacom_intuos_inout(struct urb *urb)
 			default: /* Unknown tool */
 				wacom->tool[idx] = BTN_TOOL_PEN; break;
 		}
-		input_report_abs(dev, ABS_MISC, wacom->id[idx]); /* report tool id */
-		input_report_key(dev, wacom->tool[idx], 1]);
-		input_event(dev, EV_MSC, MSC_SERIAL, wacom->serial[idx]);
+		if(!((wacom->tool[idx] == BTN_TOOL_LENS) && 
+				(strstr(wacom->features->name, "Intuos3 12x12") 
+					|| strstr(wacom->features->name, "Intuos3 12x19"))))
+		{
+			input_report_abs(dev, ABS_MISC, wacom->id[idx]); /* report tool id */
+			input_report_key(dev, wacom->tool[idx], 1]);
+			input_event(dev, EV_MSC, MSC_SERIAL, wacom->serial[idx]);
+		}
 		return 1;
 	}
 
@@ -472,7 +484,11 @@ static int wacom_intuos_inout(struct urb *urb)
 		return 1;
 	}
 
-	return 0;
+	if((wacom->tool[idx] == BTN_TOOL_LENS) && (strstr(wacom->features->name, "Intuos3 12x12") 
+			|| strstr(wacom->features->name, "Intuos3 12x19")))
+		return 1;
+	else
+		return 0;
 }
 
 static void wacom_intuos_general(struct urb *urb)
@@ -622,8 +638,8 @@ static void wacom_intuos_irq(struct urb *urb)
 			input_report_key(dev, BTN_MIDDLE, data[8] & 0x08);
 			input_report_key(dev, BTN_RIGHT,  data[8] & 0x10);
 			/* mouse wheel is positive when rolled backwards */
-			input_report_rel(dev, REL_WHEEL, (__u32)((data[8] & 0x02) >> 1)
-					 - (__u32)(data[8] & 0x01));
+			input_report_rel(dev, REL_WHEEL, (__u32)(data[8] & 0x01)
+					 - (__u32)((data[8] & 0x02) >> 1));
 
 			/* I3 2D mouse side buttons */	
 			if (strstr(wacom->features->name, "Intuos3"))
@@ -767,14 +783,22 @@ struct wacom_features wacom_features[] = {
 	/* 35 */ { "Wacom Intuos3 9x12",  10,  60960, 45720,  1023, 15,
 			wacom_intuos_irq, WACOM_INTUOS_BITS, WACOM_INTUOS3_ABS,
 			WACOM_INTUOS_REL, WACOM_INTUOS3_BUTTONS, WACOM_INTUOS3_TOOLS },
-	/* 36 */ { "Wacom Intuos3 6x11",  10,  54204, 31750,  1023, 15,
+	/* 36 */ { "Wacom Intuos3 12x12", 10,  60960, 60960,  1023, 15,
+			wacom_intuos_irq, WACOM_INTUOS_BITS, WACOM_INTUOS3_ABS,
+			WACOM_INTUOS_REL, WACOM_INTUOS3_BUTTONS, WACOM_INTUOS3_TOOLS },
+	/* 37 */ { "Wacom Intuos3 12x19", 10,  97536, 60960,  1023, 15,
+			wacom_intuos_irq, WACOM_INTUOS_BITS, WACOM_INTUOS3_ABS,
+			WACOM_INTUOS_REL, WACOM_INTUOS3_BUTTONS, WACOM_INTUOS3_TOOLS },
+	/* 38 */ { "Wacom Intuos3 6x11",  10,  54204, 31750,  1023, 15,
 			wacom_intuos_irq, WACOM_INTUOS_BITS, WACOM_INTUOS3_ABS,
 			WACOM_INTUOS_REL, WACOM_INTUOS3_BUTTONS, WACOM_INTUOS3_TOOLS },
 	/* Protocol 5 Cintiq */
-	/* 37 */ { "Wacom Cintiq 21UX",   10,  87200, 65600,  1023, 15,
+	/* 39 */ { "Wacom Cintiq 21UX",   10,  87200, 65600,  1023, 15,
 			wacom_intuos_irq, WACOM_INTUOS_BITS, WACOM_INTUOS3_ABS,
 			WACOM_INTUOS_REL, WACOM_INTUOS3_BUTTONS, WACOM_INTUOS3_TOOLS },
-	/* 38 */ { "Wacom DTF700",         8,   6858,  5506,   511, 32,
+	/* 40 */ { "Wacom DTF720",         8,   6858,  5506,   511, 32,
+			wacom_pl_irq, 0,  0, 0, 0 },
+	/* 41 */ { "Wacom DTF521",         8,   6282,  4762,   511, 32,
 			wacom_pl_irq, 0,  0, 0, 0 },
 
 	{ NULL , 0 }
@@ -817,9 +841,12 @@ struct usb_device_id wacom_ids[] = {
 	{ USB_DEVICE(USB_VENDOR_ID_WACOM, 0xB0), driver_info: 33 },
 	{ USB_DEVICE(USB_VENDOR_ID_WACOM, 0xB1), driver_info: 34 },
 	{ USB_DEVICE(USB_VENDOR_ID_WACOM, 0xB2), driver_info: 35 },
-	{ USB_DEVICE(USB_VENDOR_ID_WACOM, 0xB5), driver_info: 36 },
-	{ USB_DEVICE(USB_VENDOR_ID_WACOM, 0x3F), driver_info: 37 },
-	{ USB_DEVICE(USB_VENDOR_ID_WACOM, 0xC0), driver_info: 38 },
+	{ USB_DEVICE(USB_VENDOR_ID_WACOM, 0xB3), driver_info: 36 },
+	{ USB_DEVICE(USB_VENDOR_ID_WACOM, 0xB4), driver_info: 37 },
+	{ USB_DEVICE(USB_VENDOR_ID_WACOM, 0xB5), driver_info: 38 },
+	{ USB_DEVICE(USB_VENDOR_ID_WACOM, 0x3F), driver_info: 39 },
+	{ USB_DEVICE(USB_VENDOR_ID_WACOM, 0xC0), driver_info: 40 },
+	{ USB_DEVICE(USB_VENDOR_ID_WACOM, 0xC4), driver_info: 41 },
 
 	/* some Intuos2 6x8's erroneously report as 0x47;
 	 * multiple confirmed examples exist. */
@@ -881,8 +908,9 @@ static void *wacom_probe(struct usb_device *dev, unsigned int ifnum, const struc
 	wacom->dev.evbit[0] |= BIT(EV_KEY) | BIT(EV_ABS) | BIT(EV_MSC) |
 			wacom->features->evbit;
 	wacom->dev.absbit[0] |= BIT(ABS_X) | BIT(ABS_Y) | BIT(ABS_PRESSURE) |
-			BIT(ABS_DISTANCE) | BIT(ABS_WHEEL) | BIT(ABS_MISC) |
+			BIT(ABS_DISTANCE) | BIT(ABS_WHEEL) |
 			wacom->features->absbit;
+	wacom->dev.absbit[LONG(ABS_MISC)] |= BIT(ABS_MISC);
 	wacom->dev.relbit[0] |= wacom->features->relbit;
 	wacom->dev.keybit[LONG(BTN_LEFT)] |= BIT(BTN_LEFT) | BIT(BTN_RIGHT) |
 			BIT(BTN_MIDDLE) | wacom->features->btnbit;
