@@ -2,7 +2,7 @@
 ** wacusb.c
 **
 ** Copyright (C) 2002 - 2004 - John E. Joganic
-** Copyright (C) 2003 - 2005 - Ping Cheng
+** Copyright (C) 2003 - 2006 - Ping Cheng
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -196,8 +196,9 @@ static int USBIdentifyModel(USBTABLET* pUSB);
 		{ "MODEL_PL800",   "Wacom PL800",    6, 0x35 },
 		{ "MODEL_PL700",   "Wacom PL700",    7, 0x37 },
 		{ "MODEL_PL510",   "Wacom PL510",    8, 0x38 },
-		{ "MODEL_DTU710",  "Wacom PL710",   10, 0x39 },
-		{ "MODEL_DTF720",  "Wacom DTF720",  11, 0xC0 },
+		{ "MODEL_DTU710",  "Wacom PL710",    9, 0x39 },
+		{ "MODEL_DTF720",  "Wacom DTF720",  10, 0xC0 },
+		{ "MODEL_DTF521",  "Wacom DTF521",  11, 0xC4 },
 		{ NULL }
 	};
 
@@ -243,10 +244,12 @@ static int USBIdentifyModel(USBTABLET* pUSB);
 
 	static USBSUBTYPE xIntuos3[] =
 	{
-		{ "PTZ-430",  "Wacom Intuos3 4x5",  1, 0xB0 },
-		{ "PTZ-630",  "Wacom Intuos3 6x8",  2, 0xB1 },
-		{ "PTZ-930",  "Wacom Intuos3 9x12", 3, 0xB2 },
-		{ "PTZ-631W", "Wacom Intuos3 6x8",  4, 0xB5 },
+		{ "PTZ-430",   "Wacom Intuos3 4x5",   1, 0xB0 },
+		{ "PTZ-630",   "Wacom Intuos3 6x8",   2, 0xB1 },
+		{ "PTZ-930",   "Wacom Intuos3 9x12",  3, 0xB2 },
+		{ "PTZ-1230",  "Wacom Intuos3 12x12", 4, 0xB3 },
+		{ "PTZ-1231W", "Wacom Intuos3 12x19", 5, 0xB4 },
+		{ "PTZ-631W",  "Wacom Intuos3 6x11",  6, 0xB5 },
 		{ NULL }
 	};
 
@@ -483,8 +486,6 @@ static int USBFindModel(USBTABLET* pUSB, unsigned int uVendor,
 	return 0;
 }
 
-
-
 static int USBIdentifyModel(USBTABLET* pUSB)
 {
 	int nCnt;
@@ -512,6 +513,22 @@ static int USBIdentifyModel(USBTABLET* pUSB)
 	}
 	assert(nCnt == sizeof(evbits));
 
+	/* key events */
+	if (ISBITSET(evbits,EV_KEY))
+	{
+		nCnt = ioctl(pUSB->fd,EVIOCGBIT(EV_KEY,sizeof(keybits)),keybits);
+		if (nCnt < 0)
+		{
+			WacomLog(pUSB->hEngine,WACOMLOGLEVEL_ERROR,
+				"Failed to CGBIT key: %s",strerror(errno));
+			return 1;
+		}
+		assert(nCnt == sizeof(keybits));
+
+		pUSB->state.uValid |= BIT(WACOMFIELD_PROXIMITY) | 
+					BIT(WACOMFIELD_TOOLTYPE);
+	}
+
 	/* absolute events */
 	if (ISBITSET(evbits,EV_ABS))
 	{
@@ -537,6 +554,7 @@ static int USBIdentifyModel(USBTABLET* pUSB)
 			USBGetRange(pUSB,absbits,ABS_WHEEL,WACOMFIELD_ABSWHEEL) ||
 			USBGetRange(pUSB,absbits,ABS_THROTTLE,WACOMFIELD_THROTTLE))
 			return 1;
+
 	}
 
 	/* relative events */
@@ -560,6 +578,7 @@ static int USBIdentifyModel(USBTABLET* pUSB)
 	}
 
 	/* key events */
+	pUSB->nToolProx = 0;
 	if (ISBITSET(evbits,EV_KEY))
 	{
 		nCnt = ioctl(pUSB->fd,EVIOCGBIT(EV_KEY,sizeof(keybits)),keybits);
@@ -606,6 +625,8 @@ static int USBIdentifyModel(USBTABLET* pUSB)
 	{
 		case WACOMDEVICE_INTUOS:
 		case WACOMDEVICE_INTUOS2:
+		case WACOMDEVICE_INTUOS3:
+		case WACOMDEVICE_CINTIQV5:
 			pUSB->state.uValid |= BIT(WACOMFIELD_SERIAL);
 		default: ;
 	}
@@ -703,7 +724,7 @@ static int USBParseMSC(USBTABLET* pUSB, struct input_event* pEv)
 
 static int USBParseKEY(USBTABLET* pUSB, struct input_event* pEv)
 {
-	int i, button=-1, tool=0;
+	int button=-1, tool=0;
 	switch (pEv->code)
 	{
 		case BTN_LEFT: button = WACOMBUTTON_LEFT; break;
@@ -749,38 +770,36 @@ static int USBParseKEY(USBTABLET* pUSB, struct input_event* pEv)
 		if (pEv->value)
 		{
 			/* no prior tool in proximity */
-			if (!pUSB->nToolProx)
+			if (!(pUSB->nToolProx & BIT(tool)))
 			{
 				pUSB->state.values[WACOMFIELD_PROXIMITY].nValue = 1;
-				pUSB->state.values[WACOMFIELD_TOOLTYPE].nValue = tool;
 			}
 
 			/* remember tool in prox */
 			pUSB->nToolProx |= BIT(tool);
+			pUSB->state.values[WACOMFIELD_TOOLTYPE].nValue = tool;
 		}
 
 		/* otherwise, going out of proximity */
 		else
 		{
 			/* forget tool in prox */
-			pUSB->nToolProx &= ~BIT(tool);
+			if (pUSB->nToolProx & BIT(tool))
+			{
+				pUSB->nToolProx &= ~BIT(tool);
+			}
+
+			/* single input */
+			if (!(pUSB->state.uValid & BIT(WACOMFIELD_SERIAL)))
+				pUSB->nToolProx = 0;
+
+			pUSB->state.values[WACOMFIELD_PROXIMITY].nValue = 0;
+			pUSB->state.values[WACOMFIELD_TOOLTYPE].nValue = 0;
 
 			/* nobody left? out of proximity */
 			if (!pUSB->nToolProx)
 				memset(pUSB->state.values, 0,
 						pUSB->state.uValueCnt * sizeof(WACOMVALUE));
-
-			/* otherwise, switch to next tool */
-			else
-			{
-				for (i=WACOMTOOLTYPE_PEN; i<WACOMTOOLTYPE_MAX; ++i)
-				{
-					if (pUSB->nToolProx & BIT(i))
-					{
-						pUSB->state.values[WACOMFIELD_TOOLTYPE].nValue = i;
-					}
-				}
-			}
 		} /* out of prox */
 	} /* end if tool */
 
