@@ -1,5 +1,6 @@
 /*
- * Copyright 1995-2004 by Frederic Lepied, France. <Lepied@XFree86.org>
+ * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
+ * Copyright 2002-2005 by Ping Cheng, Wacom. <pingc@wacom.com>
  *                                                                            
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is  hereby granted without fee, provided that
@@ -137,6 +138,7 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 					/* disabled (=0) when user doesn't want to move the */
 					/* cursor from one screen to another screen */
 	common->wcmTPCButton = 0;       /* set Tablet PC button on/off, default is off */
+	common->wcmRotate = ROTATE_NONE; /* default tablet rotation to off */
 	return local;
 }
 
@@ -352,7 +354,7 @@ static InputInfoPtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 
 	/* Optional configuration */
 
-	xf86Msg(X_CONFIG, "%s serial device is %s\n", dev->identifier,
+	xf86Msg(X_CONFIG, "%s device is %s\n", dev->identifier,
 			common->wcmDevice);
 
 	gWacomModule.debugLevel = xf86SetIntOption(local->options,
@@ -381,9 +383,9 @@ static InputInfoPtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 			priv->flags |= ABSOLUTE_FLAG;
 	}
 
-	/* pad always in relative mode since it doesn't move the cursor */
+	/* pad reports absolute value and never moves the cursor */
 	if (IsPad(priv)) 
-			priv->flags &= ~ABSOLUTE_FLAG;
+		priv->flags |= ABSOLUTE_FLAG;
 
 	xf86Msg(X_CONFIG, "%s is in %s mode\n", local->name,
 		(priv->flags & ABSOLUTE_FLAG) ? "absolute" : "relative");
@@ -399,8 +401,6 @@ static InputInfoPtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 			dev->identifier);
 	}
 
-	common->wcmRotate=ROTATE_NONE;
-
 	s = xf86FindOptionValue(local->options, "Rotate");
 
 	if (s)
@@ -409,6 +409,7 @@ static InputInfoPtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 			common->wcmRotate=ROTATE_CW;
 		else if (xf86NameCmp(s, "CCW") ==0)
 			common->wcmRotate=ROTATE_CCW;
+		xf86Msg(X_CONFIG, "WACOM: Rotation is set to %s\n", s);
 	}
 
 	common->wcmSuppress = xf86SetIntOption(local->options, "Suppress",
@@ -590,6 +591,23 @@ static InputInfoPtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 			xf86Msg(X_CONFIG, "%s: Tablet PC buttons on \n", common->wcmDevice);
 	}
 
+	/* Turn on/off Gimp support in a multimonitor setup */
+	if ( !common->wcmGimp )
+	{
+		common->wcmGimp = xf86SetBoolOption(local->options, "Gimp", 1);
+		if ( !common->wcmGimp )
+			xf86Msg(X_CONFIG, "%s: Gimp multimonitor mapping isn't supported \n", common->wcmDevice);
+	}
+
+	/* Cursor stays in one monitor in a multimonitor setup */
+	if ( !common->wcmMMonitor )
+	{
+		common->wcmMMonitor = xf86SetBoolOption(local->options, "MMonitor", 1);
+		if ( !common->wcmMMonitor )
+			xf86Msg(X_CONFIG, "%s: Cursor will stay in one monitor \n", common->wcmDevice);
+	}
+
+
 	for (i=0; i<16; i++)
 	{
 		sprintf(b, "Button%d", i+1);
