@@ -1,6 +1,6 @@
 /*
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
- * Copyright 2002-2005 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
+ * Copyright 2002-2006 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
  *                                                                            
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is  hereby granted without fee, provided that
@@ -26,6 +26,24 @@
 #include "wcmFilter.h"
 
 #ifdef LINUX_INPUT
+
+/* support for compiling module on kernels older than 2.6 */
+#ifndef EV_MSC
+#define EV_MSC 0x04
+#endif
+
+#ifndef MSC_SERIAL
+#define MSC_SERIAL 0x00
+#endif
+
+#ifndef EV_SYN
+#define EV_SYN 0x00
+#endif
+
+#ifndef SYN_REPORT
+#define SYN_REPORT 0
+#endif
+
 static Bool usbDetect(LocalDevicePtr);
 static Bool usbInit(LocalDevicePtr pDev);
 
@@ -277,7 +295,7 @@ static Bool usbDetect(LocalDevicePtr local)
 				(version >> 8) & 0xff, version & 0xff);
 */
 		/* Try to grab the event device so that data don't leak to /dev/input/mice */
-#ifdef EV_SYN
+#ifdef EVIOCGRAB
 		SYSCALL(err = ioctl(local->fd, EVIOCGRAB, (pointer)1));
 
 		if (err < 0) 
@@ -352,9 +370,10 @@ static Bool usbInit(LocalDevicePtr local)
 			case 0x33: /* PL600SX */
 			case 0x34: /* PL550 */
 			case 0x35: /* PL800 */
-			case 0x37: /* PL700 */
+			case 0x37: /* PTU700 */
 			case 0x38: /* PL510 */
 			case 0xC0: /* DTF720 */
+			case 0xC4: /* DTF521 */
 				model = &usbCintiq; break;
 
 			case 0x41: /* Intuos2 4x5 */
@@ -377,6 +396,8 @@ static Bool usbInit(LocalDevicePtr local)
 			case 0xB0: /* Intuos3 4x5 */
 			case 0xB1: /* Intuos3 6x8 */
 			case 0xB2: /* Intuos3 9x12 */
+			case 0xB3: /* Intuos3 12x12 */
+			case 0xB4: /* Intuos3 12x19 */
 			case 0xB5: /* Intuos3 6x11 */
 				model = &usbIntuos3; break;
 
@@ -448,6 +469,19 @@ static int usbGetRanges(LocalDevicePtr local)
 		return !Success;
 	}
 
+	/* determine if this version of the kernel uses SYN_REPORT or MSC_SERIAL
+	   to indicate end of record for a complete tablet event */
+	if (ISBITSET(ev,EV_SYN))
+	{
+		DBG(2, ErrorF("WACOM: Kernel supports SYN_REPORTs\n"));
+		common->wcmFlags |= USE_SYN_REPORTS_FLAG;
+	}
+	else
+	{
+		DBG(2, ErrorF("WACOM: Kernel doesn't support SYN_REPORTs\n"));
+		common->wcmFlags &= ~USE_SYN_REPORTS_FLAG;
+	}
+
 	/* absolute values */
 	if (ISBITSET(ev,EV_ABS))
 	{
@@ -520,7 +554,7 @@ static int usbParse(WacomCommonPtr common, const unsigned char* data)
 static void usbParseEvent(WacomCommonPtr common,
 	const struct input_event* event)
 {
-	int i, serial, channel;
+	int i, channel;
 
 	/* store events until we receive the MSC_SERIAL containing
 	 * the serial number; without it we cannot determine the
@@ -533,42 +567,54 @@ static void usbParseEvent(WacomCommonPtr common,
 		DBG(1, ErrorF("usbParse: Exceeded event queue (%d)\n",
 				common->wcmEventCnt));
 		common->wcmEventCnt = 0;
+		common->wcmLastToolSerial = 0;
 		return;
 	}
 
 	/* save it for later */
 	common->wcmEvents[common->wcmEventCnt++] = *event;
 
-	/* packet terminated by MSC_SERIAL on kernel 2.4 and SYN_REPORT on kernel 2.6 */
-	if ((event->type != EV_MSC) || (event->code != MSC_SERIAL))
+	/* Check for end of record indicator.  On 2.4 kernels MSC_SERIAL is used
+	   but on 2.6 and later kernels SYN_REPORT is used.  */
+	if ((event->type == EV_MSC) && (event->code == MSC_SERIAL))
 	{
-		/* 2.6 none serial number tools fall here */
-#ifdef EV_SYN
-		if ((event->type == EV_SYN) && (event->code == SYN_REPORT) 
-			&& (common->wcmChannelCnt == 1))
+		/* save the serial number so we can look up the channel number later */
+		common->wcmLastToolSerial = event->value;
+
+		/* if SYN_REPORT is end of record indicator, we are done */
+		if (USE_SYN_REPORTS(common))
+			return;
+
+		/* fall through to deliver the X event */
+	}
+	else if ((event->type == EV_SYN) && (event->code == SYN_REPORT))
+	{
+		/* if we got a SYN_REPORT but weren't expecting one, change over to
+		   using SYN_REPORT as the end of record indicator */
+		if (! USE_SYN_REPORTS(common))
 		{
-			usbParseChannel(common,0,0);
-			common->wcmEventCnt = 0;
+			DBG(2, ErrorF("WACOM: Got unexpected SYN_REPORT, changing mode\n"));
+
+			/* we can expect SYN_REPORT's from now on */
+			common->wcmFlags |= USE_SYN_REPORTS_FLAG;
 		}
-#else
-		if ((event->type == 0x00) && (event->code == 0x00) 
-			&& (common->wcmChannelCnt == 1))
-		{
-			usbParseChannel(common,0,0);
-			common->wcmEventCnt = 0;
-		}
-#endif
+
+		/* fall through to deliver the X event */
+	}
+	else
+	{
+		/* not an MSC_SERIAL or SYN_REPORT, bail out */
 		return;
 	}
-	/* serial number is key for channel */
-	serial = event->value;
+
+	/* figure out the channel to use based on serial number */
 	channel = -1;
 
 	/* one channel only? */
 	if (common->wcmChannelCnt == 1)
 	{
 		/* Intuos3 or Graphire4 Pad */
-		if (serial == 0xffffffff || serial == 0xf0)
+		if (common->wcmLastToolSerial == 0xffffffff || common->wcmLastToolSerial == 0xf0)
 		{
 			channel = 1;
 			(&common->wcmChannel[channel].work)->device_type = PAD_ID;
@@ -600,7 +646,7 @@ static void usbParseEvent(WacomCommonPtr common,
 		/* find existing channel */
 		for (i=0; i<common->wcmChannelCnt; ++i)
 		{
-			if (common->wcmChannel[i].work.serial_num == serial)
+			if (common->wcmChannel[i].work.serial_num == common->wcmLastToolSerial)
 			{
 				channel = i;
 				break;
@@ -632,8 +678,8 @@ static void usbParseEvent(WacomCommonPtr common,
 		}
 	}
 
-	usbParseChannel(common,channel,serial);
-
+	usbParseChannel(common,channel,common->wcmLastToolSerial);
+	common->wcmLastToolSerial = 0;
 	common->wcmEventCnt = 0;
 }
 
@@ -693,7 +739,7 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial)
 		else if (event->type == EV_REL)
 		{
 			if (event->code == REL_WHEEL)
-				ds->relwheel = event->value;
+				ds->relwheel = -event->value;
 			else
 				ErrorF("wacom: rel event recv'd (%d)!\n", event->code);
 		}
