@@ -1,7 +1,7 @@
 /* $XConsortium: xf86Wacom.c /main/20 1996/10/27 11:05:20 kaleb $ */
 /*
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org> 
- * Copyright 2002-2005 by Ping Cheng, Wacom Technology. <pingc@wacom.com>
+ * Copyright 2002-2006 by Ping Cheng, Wacom Technology. <pingc@wacom.com>
  * 
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is  hereby granted without fee, provided that
@@ -91,9 +91,12 @@
  * 2005-11-17 47-pc0.7.1-1 - Report tool serial number and ID to Xinput
  * 2005-12-02 47-pc0.7.1-2 - Grap the USB port so /dev/input/mice won't get it
  * 2005-12-21 47-pc0.7.2 - new release
+ * 2006-03-21 47-pc0.7.3 - new release
+ * 2006-03-31 47-pc0.7.3-1 - new release
+ * 2006-05-03 47-pc0.7.4 - new release
  */
 
-static const char identification[] = "$Identification: 47-0.7.2 $";
+static const char identification[] = "$Identification: 47-0.7.4 $";
 
 /****************************************************************************/
 
@@ -425,7 +428,7 @@ static void xf86WcmDevReadInput(LocalDevicePtr local)
 void xf86WcmReadPacket(LocalDevicePtr local)
 {
 	WacomCommonPtr common = ((WacomDevicePtr)(local->private))->common;
-	int len, pos, cnt, remaining;
+	int len, pos, cnt, remaining, loop;
 
 	if (!common->wcmModel) return;
 
@@ -437,12 +440,20 @@ void xf86WcmReadPacket(LocalDevicePtr local)
 		common->bufpos, remaining));
 
 	/* fill buffer with as much data as we can handle */
-	SYSCALL(len = xf86WcmRead(local->fd,
-		common->buffer + common->bufpos, remaining));
+	len = xf86WcmRead(local->fd,
+		common->buffer + common->bufpos, remaining);
 
 	if (len <= 0)
 	{
 		ErrorF("Error reading wacom device : %s\n", strerror(errno));
+		/* In case of error, we assume the device has been disconnected
+		   so we close it. We need to iterate over all wcmDevices to
+		   actually close the device. */
+		for (loop=0; loop<common->wcmNumDevices; loop++)
+		{
+			if (common->wcmDevices[loop]->fd >= 0)
+				xf86WcmDevProc(common->wcmDevices[loop]->dev,DEVICE_OFF);
+		}
 		return;
 	}
 
@@ -514,7 +525,7 @@ static void xf86WcmDevClose(LocalDevicePtr local)
 	if (num == 1)
 	{
 		DBG(1,ErrorF("Closing device; uninitializing.\n"));
-		SYSCALL(xf86WcmClose(local->fd));
+		xf86WcmClose(local->fd);
 		common->wcmInitialized = FALSE;
 	}
 
@@ -622,9 +633,10 @@ static int xf86WcmDevProc(DeviceIntPtr pWcm, int what)
 			DBG(1, ErrorF("xf86WcmProc fd=%d pWcm=%p what=ON\n",
 				local->fd, (void *)pWcm));
 
-			if ((local->fd < 0) && (!xf86WcmDevOpen(pWcm)))
+			/* if ((local->fd < 0) && (!xf86WcmDevOpen(pWcm))) */
+			if (!xf86WcmDevOpen(pWcm))
 			{
-				pWcm->inited = FALSE;
+				/* pWcm->inited = FALSE; */
 				return !Success;
 			}
 			xf86AddEnabledDevice(local);
@@ -1339,7 +1351,7 @@ static Bool xf86WcmInitDevice(LocalDevicePtr local)
 		if (local->fd >= 0)
 		{
 			DBG(1,ErrorF("Closing device\n"));
-			SYSCALL(xf86WcmClose(local->fd));
+			xf86WcmClose(local->fd);
 		}
 		local->fd = -1;
 		return FALSE;
